'use strict';

//Global utilities --------------------

var MDW = MDW || {};
MDW.utils = {
	Date: {
		toString: function(value) {
			if (value) {
				return new Date(value).toLocaleString();
			}
			else {
				return "";
			}
		},
		toTimeString: function(value) {
			if (value) {
				return new Date(value).toLocaleTimeString();
			}
			else {
				return "";
			}
		},
		toDurationString: function(duration) {
			if (duration && duration.length != 0) {
				
				// hours
				var start = duration.indexOf("T");
				if (start >= 0) {
					start++;
					var end = duration.indexOf("H", start);
					if (end > start) {
						var hours = duration.substring(start, end);
						if (hours.length < 2) {
							hours = "0" + hours;
						}
						start = end + 1;
						
						// minutes
						end = duration.indexOf("M", start);
						if (end >= start) {
							var minutes = duration.substring(start, end);
							if (minutes.length < 2) {
								minutes = "0" + minutes;
							}
							start = end + 1;
							
							// seconds
							end = duration.indexOf("S", start);
							if (end >= start) {
								var seconds = duration.substring(start, end);
								
								// remove decimal
								start = seconds.indexOf(".");
								if (start > 0) {
									seconds = seconds.substring(0, start);
								}
								
								if (seconds.length < 2) {
									seconds = "0" + seconds;
								}
								
								return hours + ":" + minutes + ":" + seconds;
							}
						}
					}
				}
			}
			
			return "";
		}
	}
}


var commonApp = angular.module("commonApp", []);

commonApp.factory('Operation', 
	['$http', '$timeout',
	function($http, $timeout) {
		
	return {
		create: function() {
			var addQueryParameter = function(url, parameter) {
				if (url.indexOf("?") >= 0) {
					url += "&";
				}
				else {
					url += "?";
				}
				return url + parameter;
			}

			return {
				state: 0,
				progress: null,
				error: null,
				
				getProgress: function() {
					return this.progress;
				},
				
				getError: function() {
					return this.error;
				},
				
				isRunning: function() {
					return this.state == 1;
				},
				
				isTerminated: function() {
					return this.state == 2;
				},
				
				isSuccess: function() {
					return this.isTerminated() && !this.error;
				},
				
				isFailed: function() {
					return this.error;
				},

				get: function(config) {
					if (! config) { throw "config must not be null" }
					if (! config.url) { throw "config.url must not be null" }
					
					// IE has a very aggressive cache system. 
					// Add a timestamp in the query to be sure the request is not cached (unless caching is explicitly requested by caller).
					if (!config.cache) {
						config.url = addQueryParameter(config.url, "nocache=" + new Date().getTime());
					}
					
					config.method = "GET";
					this.execute(config)
				},
				
				doDelete: function(config) {
					config.method = "DELETE";
					this.execute(config)
				},
				
				execute: function(config) {
					if (! config) { throw "config must not be null" }
					if (! config.url) { throw "config.url must not be null" }
					if (! config.method) { throw "config.method must not be null" }
					var operation = this;
					
					var successHandler = function(response) {
						operation.notifySuccess();
						if (config.success) {
							config.success.call(operation, response);
						}
					};
					
					var errorHandler = function(response) {
						// call custom code before error handling, to allow customization of error
						if (config.error) {
							config.error.call(operation, response);
						}
						operation.notifyError(response);
					};
				
					operation.notifyStart(config.task);
					$http(config).then(successHandler, errorHandler);
				},
				
				notifyStart: function(taskName) {
					this.state = 1 /* running */;
					this.progress = taskName;
					this.error = null;
				},
				
				notifySuccess: function() {
					this.state = 2 /* terminated */;
					this.progress = null;
					this.error = null;
				},
				
				notifyError: function(error) {
					this.state = 2 /* terminated */;
					this.progress = null;
					this.error = {
						status: error.status,
						statusText: error.statusText ? " " + error.statusText : "HTTP operation failed - HTTP " + error.status
					};
				},
			}
		}
	}
}]);

commonApp.factory("Resource", 
	["Operation", 
	function(Operation) {
		
	return {
		create: function() {
			var resource = {
				operation: Operation.create(),
				data: {},
				url: null
			};
			var listeners = [];
			
			resource.addListener = function(listener) {
				listeners.push(listener);
			};
			
			resource.getUrl = function() {
				return resource.url;
			};
			
			resource.createResources = function(dataArray, factory) {
				var result = [];
				if (dataArray) {
					angular.forEach(dataArray, function(data) {
						var object = factory.create();
						object.loadData(data);
						result.push(object);
					});
				}
				return result;
			}

			resource.loadResource = function(config) {
				// listeners
				angular.forEach(listeners, function(listener) {
					if (listener.loadResource) {
						listener.loadResource.call(resource, config);
					}
				});
				
				// load resource
				resource.url = config.url;
				resource.operation.get({
					url: config.url,
					params: config.params,
					headers: config.headers,
					cache: config && config.cache,
					success: function(response) {
						resource.loadResponse(response);
						if (config.success) {
							config.success.call(resource, response);
						}
					},
					error: function(error) {
						if (config.error) {
							config.error.call(resource, response);
						}
					}
				});
			};
			
			resource.loadResponse = function(response) {
				this.loadData(response.data);
				
				// listeners
				angular.forEach(listeners, function(listener) {
					if (listener.loadResponse) {
						listener.loadResponse.call(resource, response);
					}
				});
			};
			
			resource.loadData = function(data) {
				resource.data = data;
				
				// listeners
				angular.forEach(listeners, function(listener) {
					if (listener.loadData) {
						listener.loadData.call(resource, data);
					}
				});
			}
			
			resource.deleteResource = function(config) {
				resource.operation.doDelete({
					url: resource.getUrl(),
					task: config.task,
					params: config.params,
					success: function(response) {
						if (config.success) {
							config.success.call(resource, response);
						}
					},
					error: function(response) {
						if (config.error) {
							config.error.call(resource, response);
						}
					}
				});
			};

			return resource;
		}
	}
}]);

commonApp.factory("ServerState", 
	["Resource",
	function(Resource) {
		
	return {
		create: function() {
			var resource = Resource.create();
			
			resource.isActive = function() { return resource.data.state && resource.data.state.value == "active"; };
			resource.isSuspended = function() { return resource.data.state && resource.data.state.value == "suspended"; };
			resource.isMaintenance = function() { return resource.data.state && resource.data.state.value == "maintenance"; };
			resource.isResumeOnRestart = function() { return resource.data.state != undefined && resource.data.state.resumeOnRestart == "true" };
			
			resource.getRunningLaunchCount = function() { return resource.data.launch && resource.data.launch.running ? parseInt(resource.data.launch.running) : 0; };
			resource.getWaitingLaunchCount = function() { return resource.data.launch && resource.data.launch.waiting ? parseInt(resource.data.launch.waiting) : 0; };
			
			return resource;
		}
	}
}]);