'use strict';

var launchesApp = angular.module("launchesApp", [
     "commonApp",
     "launchApp",
     "ui.bootstrap"
]);

launchesApp.factory("Service", 
	["Resource", 
	function(Resource) {
		
	return {
		create: function() {
			var resource = Resource.create();
			
			resource.getId = function() { return this.data.id; };
			resource.getName = function() { return this.data.name; };

			return resource;
		}
	}
}]);

launchesApp.factory("Services", 
	["Resource", "Service", 
	function(Resource, Service) {
		
	return {
		create: function() {
			var resource = Resource.create();
			var services = {};
			
			resource.addListener({
				loadData: function(data) {
					services = {};
					angular.forEach(data.services, function(serviceData) {
						var service = Service.create();
						service.loadData(serviceData);
						services[service.getId()] = service;
					});
				}
			});
			
			resource.getService = function(id) {
				var service = services[id];
				if (service) {
					return service;
				}
				else {
					return Service.create();
				}
			};

			return resource;
		}
	}
}]);

launchesApp.factory("LaunchSummary", 
	["$timeout", "Launch", "Resource", 
	function($timeout, Launch, Resource) {
		
	return {
		create: function() {
			var resource = Resource.create();
			var selected = false;
			var hovered = false;
			var details = null;
			var refreshDetailsPromise = null;
			
			resource.addListener({
				loadData: function(data) {
					// compute duration only once, when the launch is loaded
					if (data.startTime) {
						var duration = new Date() - new Date(data.startTime);
						var sec_num = parseInt(duration, 10) / 1000;
						var hours   = Math.floor(sec_num / 3600);
						var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
						var seconds = sec_num - (hours * 3600) - (minutes * 60);
						data.duration = "P0DT" + hours + "H" + minutes + "M" + seconds + "S";
					}
				}
			});

			resource.getId = function() { return this.data.id; };
			resource.getUrl = function() { return this.data.uri; };
			resource.getServiceId = function() { return this.data.service; };
			
			resource.getRequestNumber = function() {
				return resource.data.requestNumber;
			};
			
			resource.getRequestTime = function() {
				if (resource.data.requestTime) {
					return {
						getLabel: function() { return MDW.utils.Date.toTimeString(resource.data.requestTime); },
						getTooltip: function() {  return "Launch requested on " + MDW.utils.Date.toString(resource.data.requestTime); }
					}
				}
				else {
					return {
						getLabel: function() { return ""; },
						getTooltip: function() {  return null; }
					}
				}
			};
			
			resource.getState = function() {
				if (resource.isCanceled()) {
					return {
						getLabel: function() { return "Canceling"; },
						getTooltip: function() { return "Waiting for the service to acknowledge the cancel request..."; },
						getIcon: function() { return "glyphicon-remove-circle"; },
						getColor: function() { return "list-group-item-danger"; }
					};
				}
				else if (resource.isWaiting()) {
					return {
						getLabel: function() { return "Pending"; },
						getTooltip: function() { return null; },
						getIcon: function() { return "glyphicon-time"; },
						getColor: function() { return ""; }
					};
				}
				else {
					return {
						getLabel: function() { return "Running"; },
						getTooltip: function() { return "Launch duration " + MDW.utils.Date.toDurationString(resource.data.duration); },
						getIcon: function() { return "glyphicon-play-circle"; },
						getColor: function() { return "list-group-item-warning"; }
					};
				}
			};
			
			resource.isSelected = function() {
				return selected;
			};
			
			resource.setSelected = function(value) {
				selected = value;
			}
			
			resource.toggleSelected = function() {
				selected = !selected;
			};
			
			resource.isHovered = function() {
				return hovered;
			};
			
			resource.setHovered = function(value) {
				hovered = value;
			};
			
			resource.isCanceled = function() {
				return resource.data.state == "canceled";
			};
			
			resource.isWaiting = function() {
				return resource.data.state == "waiting";
			};
			
			resource.isDetailed = function() {
				return details != null;
			};
			
			resource.getDetails = function() {
				return details;
			};
			
			resource.toggleDetails = function() {
				var cancelRefresh = function() {
					if (refreshDetailsPromise) {
						$timeout.cancel(refreshDetailsPromise)
						refreshDetailsPromise = null;
					}
				};
				
				if (resource.isDetailed()) {
					details = null;
					cancelRefresh();
				}
				else {
					var reloadDetails = function() {
						details.loadResource({
							url: resource.getUrl()
						});
					};
					details = Launch.create();
					details.addListener({
						loadData: function() {
							cancelRefresh();
							if (! details.isTerminated()) {
								refreshDetailsPromise = $timeout(reloadDetails, 1000);
							}
						}
					});
					reloadDetails();
				}
			};

			resource.cancel = function() {
				
				// request cancel
				resource.deleteResource({});
				
				// refresh UI instantly
				resource.data.state = "canceled";
				resource.setSelected(false);
			};

			return resource;
		}
	}
}]);

launchesApp.factory("LaunchQueue", 
	["LaunchSummary", "Resource", 
	function(LaunchSummary, Resource) {
		
	return {
		create: function() {
			var resource = Resource.create();
			var launches = [];
			
			resource.addListener({
				loadData: function(data) {
					// keep track of existing launches
					// (don't want to keep creating new launches, as we want to maintain a selection state)
					var previousLauches = {};
					angular.forEach(launches, function(launch) {
						previousLauches[launch.getId()] = launch;
					});
					
					launches = [];
					angular.forEach(data.launches, function(launchData) {
						// reuse existing launch instance, if any
						var launch = previousLauches[launchData.id];
						if (! launch) {
							launch = LaunchSummary.create();
						}
						launch.loadData(launchData);
						launches.push(launch);
					});
				}
			});
			
			resource.isEmpty = function() {
				return launches.length == 0;
			};
			
			resource.getLaunches = function() {
				return launches;
			};
			
			resource.hasSelection = function() {
				var result = false;
				angular.forEach(launches, function(launch) {
					result = result || launch.isSelected();
				});
				return result;
			};
			
			resource.deselectAll = function() {
				angular.forEach(launches, function(launch) {
					launch.setSelected(false);
				});
			};
			
			resource.getSelectedLaunches = function() {
				var result = [];
				angular.forEach(launches, function(launch) {
					if (launch.isSelected()) {
						result.push(launch);
					}
				});
				return result;
			};

			return resource;
		}
	}
}]);

launchesApp.controller("LaunchesCtrl", 
	["$scope", "$http", "$timeout", "$uibModal", "LaunchQueue", "Services", "ServerState",
	function($scope, $http, $timeout, $uibModal, LaunchQueue, Services, ServerState) {
	
	var refreshPromise = null;
	$scope.services = Services.create();
	$scope.queue = LaunchQueue.create();
	$scope.state = ServerState.create();
	
	// auto-refresh as long as the queue is not empty
	$scope.queue.addListener({
		loadData: function() {
			if (! $scope.queue.isEmpty()) {
				refreshPromise = $timeout($scope.actions.refresh.run, 3000);
			}
		}
	});
	
	$scope.getLaunchService = function(launch) {
		return $scope.services.getService(launch.getServiceId());
	};
	
	var requestLaunchesCancel = function(launches) {
		$uibModal.open({
			templateUrl: "templates/confirm-delete-launches-dialog.html",
			controller: "DeleteLaunchesCtrl",
			resolve: {
				launches: function() { return launches; },
				presentation: function() {
					return {
						getLaunchService: function(launch) { 
							return $scope.getLaunchService(launch);
						}
					}
				}
			}
		});
	};
	
	var requestLaunchesReorder = function(launches, direction) {
		if (!direction || !launches || !launches.length) {
			return;
		}
		
		var ids = [];
		angular.forEach(launches, function(launch) {
			ids.push(launch.getId());
		});
		
		$http({
			method: "PUT",
			url: $scope.queue.getUrl(),
			params: {
				direction: direction,
				id: ids
			}
		}).then($scope.actions.refresh.run);
	};
	
	$scope.actions = {
		refresh: {
			isDisabled: function() {
				return $scope.queue.operation.isRunning();
			},
			run: function() {
				if ($scope.actions.refresh.isDisabled()) {
					return;
				}
				if (refreshPromise) {
					$timeout.cancel(refreshPromise);
					refreshPromise = null;
				}
				$scope.queue.loadResource({url: "../../launch"});
				$scope.state.loadResource({url: "../../server/state"});
			}
		},
		cancelSelectedLaunches: {
			isDisabled: function() {
				var enabled = false;
				angular.forEach($scope.queue.getSelectedLaunches(), function(launch) {
					enabled = enabled || !launch.isCanceled();
				});
				return !enabled;
			},
			run: function() {
				if ($scope.actions.cancelSelectedLaunches.isDisabled()) {
					return;
				}
				requestLaunchesCancel($scope.queue.getSelectedLaunches());
			}
		},
		cancelHoveredLaunch: {
			isDisabled: function(launch) {
				return launch.isCanceled();
			},
			run: function(launch) {
				if ($scope.actions.cancelHoveredLaunch.isDisabled(launch)) {
					return;
				}
				requestLaunchesCancel([ launch ]);
			}
		},
		reorderSelectedLaunches: {
			isDisabled: function() {
				var enabled = false;
				angular.forEach($scope.queue.getSelectedLaunches(), function(launch) {
					enabled = enabled || launch.isWaiting();
				});
				return !enabled;
			},
			run: function(direction) {
				if ($scope.actions.reorderSelectedLaunches.isDisabled()) {
					return;
				}
				requestLaunchesReorder($scope.queue.getSelectedLaunches(), direction);
			}
		},
		reorderHoveredLaunch: {
			isDisabled: function(launch) {
				return !launch.isWaiting();
			},
			run: function(launch, direction) {
				if ($scope.actions.reorderHoveredLaunch.isDisabled(launch)) {
					return;
				}
				requestLaunchesReorder([ launch ], direction);
			}
		}
	};
	
	// initialization
	$scope.services.loadResource({
		url: "../../service",
		cache: true
	});
	$scope.actions.refresh.run();
}]);

launchesApp.controller("DeleteLaunchesCtrl", 
	["$scope", "$uibModalInstance", "launches", "presentation",
	function($scope, $uibModalInstance, launches, presentation) {
		
	$scope.launches = launches;
	$scope.presentation = presentation;
	
	$scope.ok = function() {
		angular.forEach(launches, function(launch) {
			launch.cancel();
		});
		
		$uibModalInstance.close();
	};
	
	$scope.cancel = function() {
		$uibModalInstance.dismiss('cancel');
	};
}]);